

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;

import org.junit.Test;

public class PhotoLibraryTests {	

	@Test(timeout=100)
	public void constructor() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		assertEquals("PhotoLibrary's constructor failed to initialize name (getter did not return expected value)", "Arphaxad", b.getName());
		assertEquals("PhotoLibrary's constructor failed to initialize id (getter did not return expected value)", 14, b.getId());
		assertEquals("PhotoLibrary's constructor failed to initialize photos (getter did not return expected value)", new ArrayList<Photograph>(), b.getPhotos());
	}

	@Test(timeout=100)
	public void setName() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		b.setName("Peleg");
		assertEquals("PhotoLibrary.setName() failed (getter did not return expected value)", "Peleg", b.getName());
	}


	@Test(timeout=100)
	public void eraseEmpty() {
		PhotoLibrary b = new PhotoLibrary("Peleg", 17);
		assertFalse("PhotoLibrary.erasePhoto() failed (edge case)", b.erasePhoto(new Photograph("T","A")));
	}
	@Test(timeout=100)
	public void eraseOnly() {
		PhotoLibrary b = new PhotoLibrary("Peleg", 17);
		b.getPhotos().add(new Photograph("T", "A"));
		assertTrue("PhotoLibrary.erasePhoto() failed (erasePhoto() returned wrong result) (also check getPhotos())", b.erasePhoto(new Photograph("T","A")));
		assertEquals("PhotoLibrary.erasePhoto() failed (something wasn't removed) (also check getPhotos())", new ArrayList<Photograph>(), b.getPhotos());
	}
	@Test(timeout=100)
	public void eraseThere() {
		PhotoLibrary b = new PhotoLibrary("Peleg", 17);
		b.getPhotos().add(new Photograph("T1", "A1"));
		b.getPhotos().add(new Photograph("T2", "A2"));
		b.getPhotos().add(new Photograph("T3", "A3"));
		assertTrue(b.erasePhoto(new Photograph("T2","A2")));
		assertFalse("PhotoLibrary.erasePhoto() failed (something wasn't removed) (also check getPhotos())", b.getPhotos().contains(new Photograph("T2", "A2")));
		assertTrue("PhotoLibrary.erasePhoto() failed (removed too much) (also check getPhotos())", b.getPhotos().contains(new Photograph("T1", "A1")));
		assertTrue("PhotoLibrary.erasePhoto() failed (removed too much) (also check getPhotos())", b.getPhotos().contains(new Photograph("T3", "A3")));
	}
	@Test(timeout=100)
	public void eraseNotThere() {
		PhotoLibrary b = new PhotoLibrary("Peleg", 17);
		b.getPhotos().add(new Photograph("T1", "A1"));
		b.getPhotos().add(new Photograph("T3", "A3"));
		assertFalse(b.erasePhoto(new Photograph("T2","A2")));
		assertFalse("PhotoLibrary.erasePhoto() failed (added instead of removed) (also check getPhotos())", b.getPhotos().contains(new Photograph("T2", "A2")));
		assertTrue("PhotoLibrary.erasePhoto() failed (removed too much) (also check getPhotos())", b.getPhotos().contains(new Photograph("T1", "A1")));
		assertTrue("PhotoLibrary.erasePhoto() failed (removed too much) (also check getPhotos())", b.getPhotos().contains(new Photograph("T3", "A3")));
	}


	@Test(timeout=100)
	public void testAddNull() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
        //assertTrue("PhotoLibrary.addPhoto() failed (edge case)", b.addPhoto(null));
		if (b.addPhoto(null)) {
		    assertTrue("PhotoLibrary.addPhoto() failed (edge case)", b.getPhotos().contains(null));
		} else {
            assertFalse("PhotoLibrary.addPhoto() failed (edge case)", b.getPhotos().contains(null));
		}
	}
	@Test(timeout=100)
	public void testaddPhoto() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		Photograph k = new Photograph("some title", "some author");
		assertTrue("PhotoLibrary.addPhoto() failed", b.addPhoto(k));
		assertTrue("PhotoLibrary.addPhoto() failed (photos list doesn't include taken photo)", b.getPhotos().contains(k));
	}
	@Test(timeout=100)
	public void testAddTwoPhotographs() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		Photograph k = new Photograph("some title", "some author");
		Photograph k2 = new Photograph("another title", "same author");
		assertTrue("PhotoLibrary.addPhoto() failed", b.addPhoto(k));
		assertTrue("PhotoLibrary.addPhoto() failed", b.addPhoto(k2));
		assertTrue("PhotoLibrary.addPhoto() failed (photos list doesn't include taken photo)", b.getPhotos().contains(k));
		assertTrue("PhotoLibrary.addPhoto() failed (photos list doesn't include taken photo)", b.getPhotos().contains(k2));
	}
	@Test(timeout=100)
	public void testAddSamePhotographTwice() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		Photograph k = new Photograph("some title", "some author");
		Photograph k2 = new Photograph("another title", "same author");
		Photograph k3 = new Photograph("another title", "another author");
		assertTrue("PhotoLibrary.addPhoto() failed", b.addPhoto(k));
		assertTrue("PhotoLibrary.addPhoto() failed", b.addPhoto(k2));
		assertTrue("PhotoLibrary.addPhoto() failed", b.addPhoto(k3));
		assertFalse("PhotoLibrary.addPhoto() failed", b.addPhoto(k2));
		assertEquals("PhotoLibrary.addPhoto() failed (photos list reports wrong size after taking photos)", 3, b.getPhotos().size());
	}
	@Test(timeout=100)
	public void testAddTwoCopies() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		Photograph k = new Photograph("some title", "some author");
		Photograph k2 = new Photograph("another title", "same author");
		Photograph k3 = new Photograph("another title", "same author");
		assertTrue("PhotoLibrary.addPhoto() failed", b.addPhoto(k));
		assertTrue("PhotoLibrary.addPhoto() failed", b.addPhoto(k2));
		assertFalse("PhotoLibrary.addPhoto() failed (adding an equal photograph)", b.addPhoto(k3));
		assertEquals("PhotoLibrary.addPhoto() failed (photos list reports wrong size after taking photos)", 2, b.getPhotos().size());
	}


	@Test(timeout=100)
	public void testNum0() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		assertEquals("PhotoLibrary.numPhotographs() failed (edge case)", 0, b.numPhotographs());
	}
	@Test(timeout=100)
	public void testNum1() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		b.getPhotos().add(new Photograph("T", "A"));
		assertEquals("PhotoLibrary.numPhotographs() failed (short list) (also check getPhotos())", 1, b.numPhotographs());
	}
	@Test(timeout=100)
	public void testNum2() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		b.getPhotos().add(new Photograph("T", "A"));
		b.getPhotos().add(new Photograph("Title", "A"));
		assertEquals("PhotoLibrary.numPhotographs() failed (short list) (also check getPhotos())", 2, b.numPhotographs());
	}
	@Test(timeout=100)
	public void testNum3() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		b.getPhotos().add(new Photograph("T", "A"));
		b.getPhotos().add(new Photograph("Title", "A"));
		b.getPhotos().add(null);
		assertEquals("PhotoLibrary.numPhotographs() failed (edge case) (also check getPhotos())", 3, b.numPhotographs());
	}



	@Test(timeout=100)
	public void testhasPhotoEmpty() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		Photograph k = new Photograph("some title", "some author");
		assertFalse("PhotoLibrary.hasPhoto() failed (edge case)", b.hasPhoto(k));
	}
	@Test(timeout=100)
	public void testhasPhoto() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		Photograph k = new Photograph("some title", "some author");
		Photograph k2 = new Photograph("another title", "same author");
		Photograph k3 = new Photograph("another title", "another author");
		b.getPhotos().add(k);
		b.getPhotos().add(k2);
		b.getPhotos().add(k3);
		assertTrue("PhotoLibrary.hasPhoto() failed (also check getPhotos())", b.hasPhoto(k2));
	}
	@Test(timeout=100)
	public void testHasNotRead() {
		PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
		Photograph k = new Photograph("some title", "some author");
		Photograph k2 = new Photograph("another title", "same author");
		Photograph k3 = new Photograph("another title", "another author");
		b.getPhotos().add(k);
		b.getPhotos().add(k2);
		assertFalse("PhotoLibrary.hasPhoto() failed (also check getPhotos())", b.hasPhoto(k3));
	}


	@Test(timeout=100)
	public void equalsType() throws Exception {
        try {
            PhotoLibrary.class.getDeclaredMethod("equals", Object.class);
        } catch (NoSuchMethodException e) {
            fail("PhotoLibrary equals method not specified correctly.");
        } catch (Exception e) {
            throw e;
        }
	}

	@Test(timeout=100)
	public void equalsSame() {
		PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
		assertTrue("PhotoLibrary.equals() failed: Symmetric", b.equals(b));
	}

	@Test(timeout=100)
	public void equalsSimilar() {
		PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
		PhotoLibrary c = new PhotoLibrary("Le Petit Prince", 42);
		assertTrue("PhotoLibrary.equals() failed: Same id and name", b.equals(c));
	}

	@Test(timeout=100)
	public void equalsDifferentNames() {
		PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
		PhotoLibrary c = new PhotoLibrary("The Little Prince", 42);
		assertTrue("PhotoLibrary.equals() failed: Same id, different names", b.equals(c));
	}
	@Test(timeout=100)
	public void equalsDifferentReads() {
		PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
		PhotoLibrary c = new PhotoLibrary("Le Petit Prince", 42);
		((ArrayList<Photograph>)c.getPhotos()).add(new Photograph("Picture of a Sheep", "Me"));
		assertTrue("PhotoLibrary.equals() failed: Same id, different names", b.equals(c));
	}
	@Test(timeout=100)
	public void equalsDifferentIds() {
		PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
		PhotoLibrary c = new PhotoLibrary("Le Petit Prince", 2110);
		assertFalse("PhotoLibrary.equals() failed: Different id, same names", b.equals(c));
	}
	@Test(timeout=100)
	public void equalsEverythingDifferent() {
		PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
		PhotoLibrary c = new PhotoLibrary("The Little Prince", 2110);
		assertFalse("PhotoLibrary.equals() failed: Different id, different names", b.equals(c));
	}
	@Test(timeout=100)
	public void equalsDifferentTypes() throws Exception {
	    try {
	        PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
	        assertFalse("PhotoLibrary.equals() failed: Compare PhotoLibrary to Integer", b.equals(42));
	    } catch (ClassCastException e) {
	        fail("PhotoLibrary.equals() failed: Casting a non-PhotoLibrary to a PhotoLibrary");
	    } catch (Exception e) {
	        throw e;
	    }
	}

	@Test(timeout=100)
	public void testToString() {
		PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 402);
		b.getPhotos().add(new Photograph("Picture of a Sheep", "Me"));
		String s = b.toString();
		assertTrue("PhotoLibrary.toString must have name", s.contains("Le Petit Prince"));
		assertTrue("PhotoLibrary.toString must have id", s.contains("402"));
		assertTrue("PhotoLibrary.toString must have list of Photographs (also check getPhotos())", s.contains("Me"));
		assertTrue("PhotoLibrary.toString must have list of Photographs (also check getPhotos())", s.contains("Picture of a Sheep"));
	}




	@Test(timeout=100)
	public void testEmptyCommon() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		assertEquals("PhotoLibrary.commonPhotos() failed (edge case)", new ArrayList<Photograph>(), PhotoLibrary.commonPhotos(a,b));
	}

	@Test(timeout=100)
	public void testNoneInCommon() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		Photograph x = new Photograph("One Cello x 16", "Keating");
		Photograph y = new Photograph("One Cello x 16", "Zoe Keating");
		Photograph z = new Photograph("Palladino", "Jenkins");
		a.addPhoto(x);
		b.addPhoto(y);
		a.addPhoto(z);
		int asize = a.getPhotos().size();
		int bsize = b.getPhotos().size();
		assertEquals("PhotoLibrary.commonPhotos() failed (also check addPhoto())", new ArrayList<Photograph>(), PhotoLibrary.commonPhotos(a,b));
		assertEquals("PhotoLibrary.commonPhotos() failed (also check addPhoto())", new ArrayList<Photograph>(), PhotoLibrary.commonPhotos(a,b));
		assertEquals("PhotoLibrary.commonPhotos() failed: read lists should not change (also check addPhoto())", asize, a.getPhotos().size());
		assertEquals("PhotoLibrary.commonPhotos() failed: read lists should not change (also check addPhoto())", bsize, b.getPhotos().size());
	}
	@Test(timeout=100)
	public void testAllInCommon() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		Photograph x = new Photograph("One Cello x 16", "Keating");
		Photograph y = new Photograph("One Cello x 16", "Zoe Keating");
		Photograph z = new Photograph("Palladino", "Jenkins");
		a.addPhoto(x); b.addPhoto(z);
		a.addPhoto(y); b.addPhoto(x);
		a.addPhoto(z); b.addPhoto(y);
		int asize = a.getPhotos().size();
		int bsize = b.getPhotos().size();
		ArrayList<Photograph> common = PhotoLibrary.commonPhotos(a, b);
		assertTrue("PhotoLibrary.commonPhotos() failed (also check addPhoto())", common.contains(x));
		assertTrue("PhotoLibrary.commonPhotos() failed (also check addPhoto())", common.contains(y));
		assertTrue("PhotoLibrary.commonPhotos() failed (also check addPhoto())", common.contains(z));
		assertEquals("PhotoLibrary.commonPhotos() failed: Three shared photos, but PhotoLibrarys had more than 3 shared reported", 3, common.size());
		assertEquals("PhotoLibrary.commonPhotos() failed: photos lists should not change on call to commonPhotos() (also check getPhotos())", asize, a.getPhotos().size());
		assertEquals("PhotoLibrary.commonPhotos() failed: photos lists should not change on call to commonPhotos() (also check getPhotos())", bsize, b.getPhotos().size());
	}

	@Test(timeout=100)
	public void testOneInCommon() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		Photograph x = new Photograph("One Cello x 16", "Keating");
		Photograph y = new Photograph("One Cello x 16", "Zoe Keating");
		Photograph z = new Photograph("Palladino", "Jenkins");
		a.addPhoto(x); 
		a.addPhoto(y); b.addPhoto(y);
		b.addPhoto(z);
		int asize = a.getPhotos().size();
		int bsize = b.getPhotos().size();
		ArrayList<Photograph> common = PhotoLibrary.commonPhotos(a, b);
		assertTrue("PhotoLibrary.commonPhotos() failed (also check addPhoto())", common.contains(y));
		assertEquals("PhotoLibrary.commonPhotos() failed: PhotoLibrarys with overlapping photos should only report shared photos", 1, common.size());
		assertEquals("PhotoLibrary.commonPhotos() failed: photos lists should not change on call to commonPhotos() (also check getPhotos())", asize, a.getPhotos().size());
		assertEquals("PhotoLibrary.commonPhotos() failed: photos lists should not change on call to commonPhotos() (also check getPhotos())", bsize, b.getPhotos().size());
	}

	@Test(timeout=100)
	public void testEmptySimilarity() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		assertEquals("PhotoLibrary.similarity() failed: edge case (no photos)",0.0, PhotoLibrary.similarity(a,b), 1e-6); // 1e-6 = epsilon
		Photograph z = new Photograph("Palladino", "Jenkins");
		b.addPhoto(z);
		int asize = a.getPhotos().size();
		int bsize = b.getPhotos().size();
		assertEquals("PhotoLibrary.similarity() failed: no shared photos (also check addPhoto())", 0.0, PhotoLibrary.similarity(a,b), 1e-6); // 1e-6 = epsilon
		assertEquals("PhotoLibrary.similarity() failed: no shared photos (also check addPhoto())", 0.0, PhotoLibrary.similarity(b,a), 1e-6); // 1e-6 = epsilon
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", asize, a.getPhotos().size());
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", bsize, b.getPhotos().size());
	}
	@Test(timeout=100)
	public void testFullSimilarity() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		Photograph x = new Photograph("One Cello x 16", "Keating");
		Photograph y = new Photograph("One Cello x 16", "Zoe Keating");
		Photograph z = new Photograph("Palladino", "Jenkins");
		a.addPhoto(x); b.addPhoto(x);
		a.addPhoto(y); b.addPhoto(y);
		a.addPhoto(z); b.addPhoto(z);
		int asize = a.getPhotos().size();
		int bsize = b.getPhotos().size();
		assertEquals("PhotoLibrary.similarity() failed: shared 3 photos and with no other photos (also check addPhoto())", 1.0, PhotoLibrary.similarity(a,b), 1e-6); // 1e-6 = epsilon
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", asize, a.getPhotos().size());
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", bsize, b.getPhotos().size());
	}
	@Test(timeout=100)
	public void test2of3Similarity() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		Photograph x = new Photograph("One Cello x 16", "Keating");
		Photograph y = new Photograph("One Cello x 16", "Zoe Keating");
		Photograph z = new Photograph("Palladino", "Jenkins");
		a.addPhoto(x); b.addPhoto(x);
		a.addPhoto(y);
		b.addPhoto(z);
		int asize = a.getPhotos().size();
		int bsize = b.getPhotos().size();
		assertEquals("PhotoLibrary.similarity() failed: one shared photo, but they have additional unshared photo(s) (also check addPhoto())", 0.5, PhotoLibrary.similarity(a,b), 1e-6); // 1e-6 = epsilon
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", asize, a.getPhotos().size());
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", bsize, b.getPhotos().size());
	}
	@Test(timeout=100)
	public void testFirstFullSimilarity() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		Photograph x = new Photograph("One Cello x 16", "Keating");
		Photograph y = new Photograph("One Cello x 16", "Zoe Keating");
		Photograph z = new Photograph("Palladino", "Jenkins");
		a.addPhoto(x); b.addPhoto(x);
		a.addPhoto(y);
		int asize = a.getPhotos().size();
		int bsize = b.getPhotos().size();
		assertEquals("PhotoLibrary.similarity() failed: one-way overlap (one person has taken all photos another has) (also check addPhoto())", 1.0, PhotoLibrary.similarity(a,b), 1e-6); // 1e-6 = epsilon
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", asize, a.getPhotos().size());
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", bsize, b.getPhotos().size());
	}
	@Test(timeout=100)
	public void testSecondFullSimilarity() {
		PhotoLibrary a = new PhotoLibrary("Dana", 1);
		PhotoLibrary b = new PhotoLibrary("Chris", 2);
		Photograph x = new Photograph("One Cello x 16", "Keating");
		Photograph y = new Photograph("One Cello x 16", "Zoe Keating");
		Photograph z = new Photograph("Palladino", "Jenkins");
		a.addPhoto(x); b.addPhoto(x);
		b.addPhoto(y);
		a.addPhoto(z); b.addPhoto(z);
		int asize = a.getPhotos().size();
		int bsize = b.getPhotos().size();
		assertEquals("PhotoLibrary.similarity() failed: one-way overlap (one person has taken all photos another has) (also check addPhoto())", 1.0, PhotoLibrary.similarity(a,b), 1e-6); // 1e-6 = epsilon
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", asize, a.getPhotos().size());
		assertEquals("PhotoLibrary.similarity() failed: photos lists should not change (also check getPhotos())", bsize, b.getPhotos().size());
	}

}

