

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
public class PhotographTests {	

    @Test(timeout=100)
    public void testConstructor() {
        Photograph b = new Photograph("mypic.jpg", "Grand Canyon");
        assertEquals("Original photograph constructor (or getter) failed for caption", "Grand Canyon", b.getCaption());
        assertEquals("Original photograph constructor (or getter) failed for filename", "mypic.jpg", b.getFilename());
    }

    @Test(timeout=100)
    public void equalsType() throws Exception {
        try {
            Photograph.class.getDeclaredMethod("equals", Object.class);
        } catch (NoSuchMethodException e) {
            fail("Photograph equals method not specified correctly.");
        } catch (Exception e) {
            throw e;
        }
    }

    @Test(timeout=100)
    public void equalsSame() {
        Photograph b = new Photograph("mypic3.jpg", "Grand Canyon 2");
        assertTrue("Photograph.equals() failed: Not symmetric", b.equals(b));
    }
    @Test(timeout=100)
    public void equalsSimilar() {
        Photograph b = new Photograph("mypic3.jpg", "Grand Canyon 3");
        Photograph c = new Photograph("mypic3.jpg", "Grand Canyon 3");
        assertTrue("Photograph.equals() failed: Two photographs with same caption and file name not equal", b.equals(c));
    }


    @Test(timeout=100)
    public void equalsDifferentCaptions() {
        Photograph b = new Photograph("mypic3.jpg", "Grand Canyon 54");
        Photograph c = new Photograph("mypic3.jpg", "Grand Canyon 3");
        assertFalse("Photograph.equals() failed: Same filename, different captions", b.equals(c));
    }

    @Test(timeout=100)
    public void equalsDifferentFilenames() {
        Photograph b = new Photograph("mypic3.jpg", "Grand Canyon 3");
        Photograph c = new Photograph("mypic55.jpg", "Grand Canyon 3");
        assertFalse("Photograph.equals() failed: Same caption, different filenames", b.equals(c));
    }


    @Test(timeout=100)
    public void equalsEverythingDifferent() {
        Photograph b = new Photograph("newpic.jpg", "Grand Canyon");
        Photograph c = new Photograph("yos.jpg", "Yosemite");
        assertFalse("Photograph.equals() failed: Different caption, different filenames", b.equals(c));
    }

    @Test(timeout=100)
    public void equalsDifferentTypes() throws Exception {
        try {
            Photograph b = new Photograph("yosemite.png", "Family Vacation");
            assertFalse("Photograph.equals() failed: Compared Photograph to Integer", b.equals(42));
        } catch (ClassCastException e) {
            fail("Photograph.equals() failed: Casting a non-Photograph to a Photograph");
        } catch (Exception e) {
            throw e;
        }
    }


    @Test(timeout=100)
    public void testToString() {
        Photograph b = new Photograph("yosemite.png", "Family Vacation");
        String s = b.toString();
        assertTrue("Photograph.toString must have caption", s.contains("Family Vacation"));
        assertTrue("Photograph.toString must have filename", s.contains("yosemite.png"));
    }

}
