

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
public class WebcatAlbumTests {	

	@Test(timeout=100)
	public void testConstructor() {
		Album a = new Album("Vacation");
        assertEquals("Album constructor (or getter) failed for name", "Vacation", a.getName());
	}
    @Test(timeout=100)
    public void testConstructor2() {
        Album a = new Album("Vacation");
        assertNotNull("Album constructor did not initialize photos list (or getter failed)", a.getPhotos());
    }
    @Test(timeout=100)
    public void testAddPhotoEdge() {
        Album a = new Album("Vacation");
        Photograph b = new Photograph("mypic4.jpg", "Grand Canyon", "2019-01-11", 2);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        a.addPhoto(null);
        assertEquals("Album allowed adding null photograph", 0, a.getPhotos().size());
    }
    @Test(timeout=100)
    public void testAddPhoto() {
        Album a = new Album("Vacation");
        Photograph b = new Photograph("mypic4.jpg", "Grand Canyon", "2019-01-11", 2);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(b));
        assertEquals("Album addPhoto did not add a photo", 1, a.getPhotos().size());
    }
    @Test(timeout=100)
    public void testAddPhoto2() {
        Album a = new Album("Vacation");
        Photograph b = new Photograph("mypic4.jpg", "Grand Canyon", "2019-01-11", 2);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(b));
        assertEquals("Album addPhoto did not add a photo", 1, a.getPhotos().size());
        assertFalse("Album addPhoto incorrectly allowed a photo to be added twice", a.addPhoto(b));
        assertEquals("Album addPhoto incorrectly allowed a photo to be added twice", 1, a.getPhotos().size());
    }
    
    @Test(timeout=100)
    public void testHasPhoto() {
        Album a = new Album("Vacation");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("Album hasPhoto did not find a photo in the album", a.hasPhoto(b));
    }
    
    @Test(timeout=100)
    public void testHasPhotoEdge() {
        Album a = new Album("Vacation");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertFalse("Album hasPhoto found a photo that wasn't in the album (empty)", a.hasPhoto(b));
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(b));
        assertFalse("Album hasPhoto found a photo that wasn't in the album", a.hasPhoto(c));
        assertFalse("Album hasPhoto found a null photo", a.hasPhoto(null));
    }
    
    @Test(timeout=100)
    public void testHasPhoto2() {
        Album a = new Album("Vacations");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(d));
        assertTrue("Album hasPhoto did not find a photo in the album", a.hasPhoto(c));
        assertTrue("Album hasPhoto did not find a photo in the album", a.hasPhoto(b));
        assertTrue("Album hasPhoto did not find a photo in the album", a.hasPhoto(d));
    }
    
    @Test(timeout=100)
    public void testRemovePhoto() {
        Album a = new Album("Vacations");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("Repeated Album addPhoto didn't add the right number of photos", 3, a.getPhotos().size());
        assertTrue("Album removePhoto did not remove a photo in the album", a.removePhoto(c));
        assertFalse("Album removePhoto did not remove the right photo in the album", a.hasPhoto(c));
        assertEquals("Album removePhoto did not remove a photo in the album", 2, a.getPhotos().size());
    }
    
    @Test(timeout=100)
    public void testRemovePhotoEdge() {
        Album a = new Album("Vacation Picts");
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        assertFalse("Album removePhoto returned true when removing a photo that didn't exist", a.removePhoto(c));
        assertFalse("Album removePhoto returned true when removing null", a.removePhoto(null));
    }
    
    @Test(timeout=100)
    public void testNumPhotographs() {
        Album a = new Album("Vacation Album");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertEquals("Album numPhotographs not returning 0 if empty", 0, a.numPhotographs());
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(b));
        assertEquals("Album numPhotographs not returning correct count", 1, a.numPhotographs());
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(c));
        assertEquals("Album numPhotographs not returning correct count", 2, a.numPhotographs());
        assertTrue("Album addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("Album numPhotographs not returning correct count", 3, a.numPhotographs());
        
    }
    

	@Test(timeout=100)
	public void equalsType() throws Exception {
	    try {
	        Album.class.getDeclaredMethod("equals", Object.class);
	    } catch (NoSuchMethodException e) {
	        fail("Album equals method not specified correctly.");
	    } catch (Exception e) {
	        throw e;
	    }
	}
	
	@Test(timeout=100)
	public void equalsSame() {
        Album a = new Album("Vacation Picts");
		assertTrue("Album.equals() failed: Not symmetric", a.equals(a));
	}
	@Test(timeout=100)
	public void equalsSimilar() {
        Album a = new Album("UVA Picts");
        Album b = new Album("UVA Picts");
		assertTrue("Album.equals() failed: Two Albums with same name not equal", b.equals(a));
	}
	@Test(timeout=100)
	public void equalsDifferentTypes() throws Exception {
        try {
            Album a = new Album("Rafting");
            assertFalse("Album.equals() failed: Compared Album to String", a.equals("Rafting"));
        } catch (ClassCastException e) {
            fail("Album.equals() failed: Casting a non-Album to an Album");
        } catch (Exception e) {
            throw e;
        }
	}

	@Test(timeout=100)
	public void testToString() {        
	    Album a = new Album("Vacations");
	    Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
	    Photograph c = new Photograph("green.jpg", "Rafting", "2016-05-11", 5);
	    Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);
	    assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
	    assertTrue("Album addPhoto did not add a photo", a.addPhoto(b));
	    assertTrue("Album addPhoto did not add a photo", a.addPhoto(c));
	    assertTrue("Album addPhoto did not add a photo", a.addPhoto(d));
	    String s = a.toString();
		assertTrue("Album.toString must have name", s.contains("Vacations"));
		assertTrue("Album.toString must have photgraph filenames", s.contains("mypic45.jpg"));
        assertTrue("Album.toString must have photgraph filenames", s.contains("green.jpg"));
        assertTrue("Album.toString must have photgraph filenames", s.contains("water2.jpg"));
	}

}
