

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
public class WebcatAlbumTests {	

	@Test(timeout=100)
	public void testConstructor() {
		Album a = new Album("Vacation");
        assertEquals("Album constructor (or getter) failed for name", "Vacation", a.getName());
	}
    @Test(timeout=100)
    public void testConstructor2() {
        Album a = new Album("Vacation");
        assertNotNull("Album constructor did not initialize photos list (or getter failed)", a.getPhotos());
    }
 
    
    @Test(timeout=100)
    public void testAddPhoto() {
        Album a = new Album("Vacation");
        Photograph b = new Photograph("mypic4.jpg", "Grand Canyon", "2019-01-11", 2);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertEquals("Album's inherited addPhoto did not add a photo", 1, a.getPhotos().size());
    }
    
    @Test(timeout=100)
    public void testHasPhoto() {
        Album a = new Album("Vacation");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("Album's inherited hasPhoto did not find a photo in the album", a.hasPhoto(b));
    }
    
    @Test(timeout=100)
    public void testRemovePhoto() {
        Album a = new Album("Vacations");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("Repeated Album's inherited addPhoto didn't add the right number of photos", 3, a.getPhotos().size());
        assertTrue("Album's inherited removePhoto did not remove a photo in the album", a.removePhoto(c));
        assertFalse("Album's inherited removePhoto did not remove the right photo in the album", a.hasPhoto(c));
        assertEquals("Album's inherited removePhoto did not remove a photo in the album", 2, a.getPhotos().size());
    }
    
    @Test(timeout=100)
    public void testNumPhotographs() {
        Album a = new Album("Vacation Album");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);
        assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
        assertEquals("Album's inherited numPhotographs not returning 0 if empty", 0, a.numPhotographs());
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertEquals("Album's inherited numPhotographs not returning correct count", 1, a.numPhotographs());
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertEquals("Album's inherited numPhotographs not returning correct count", 2, a.numPhotographs());
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("Album's inherited numPhotographs not returning correct count", 3, a.numPhotographs());
        
    }
	
	@Test(timeout=100)
	public void equalsSimilar() {
        Album a = new Album("UVA Picts");
        Album b = new Album("UVA Picts");
		assertTrue("Album.equals() failed: Two Albums with same name not equal", b.equals(a));
	}
	@Test(timeout=100)
	public void equalsDifferentTypes() throws Exception {
        try {
            Album a = new Album("Rafting");
            assertFalse("Album.equals() failed: Compared Album to String", a.equals("Rafting"));
        } catch (ClassCastException e) {
            fail("Album.equals() failed: Casting a non-Album to an Album");
        } catch (Exception e) {
            throw e;
        }
	}

	@Test(timeout=100)
	public void testToString() {        
	    Album a = new Album("Vacations");
	    Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
	    Photograph c = new Photograph("green.jpg", "Rafting", "2016-05-11", 5);
	    Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);
	    assertEquals("Album constructor did not create an empty photos list", 0, a.getPhotos().size());
	    assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(b));
	    assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(c));
	    assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(d));
	    String s = a.toString();
		assertTrue("Album.toString must have name", s.contains("Vacations"));
		assertTrue("Album.toString must have photgraph filenames", s.contains("mypic45.jpg"));
        assertTrue("Album.toString must have photgraph filenames", s.contains("green.jpg"));
        assertTrue("Album.toString must have photgraph filenames", s.contains("water2.jpg"));
	}
	
    @Test(timeout=100)
    public void testAlbumGetPhotosYear() {
        Album a = new Album("MyLibrary");
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("Album's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("Album's inherited getPhotosInYear(year) did not return photos in year", 2, a.getPhotosInYear(2016).size());
        assertTrue("Album's inherited getPhotosInYear(year) did not return correct photos in year", a.getPhotosInYear(2016).contains(d));
        assertTrue("Album's inherited getPhotosInYear(year) did not return correct photos in year", a.getPhotosInYear(2016).contains(c));
    }

}
