

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.HashSet;

import org.junit.Test;

public class WebcatPhotoLibraryTests {	

    @Test(timeout=100)
    public void constructor() {
        PhotoLibrary b = new PhotoLibrary("MyLibrary", 14);
        assertEquals("PhotoLibrary's constructor failed to initialize name (getter did not return expected value)", "MyLibrary", b.getName());
        assertEquals("PhotoLibrary's constructor failed to initialize id (getter did not return expected value)", 14, b.getId());
        assertEquals("PhotoLibrary's constructor failed to initialize photos (getter did not return expected value)", new ArrayList<Photograph>(), b.getPhotos());
        assertEquals("PhotoLibrary's constructor failed to initialize albums (getter did not return expected value)", new HashSet<Album>(), b.getAlbums());
    }

    @Test(timeout=100)
    public void eraseEmpty() {
        PhotoLibrary b = new PhotoLibrary("Peleg", 17);
        assertFalse("PhotoLibrary.removePhoto() failed (edge case)", b.removePhoto(new Photograph("Caption","Filename")));
    }


    @Test(timeout=100)
    public void eraseOnly() {
        PhotoLibrary b = new PhotoLibrary("Peleg", 17);
        Photograph p = new Photograph("Caption","Filename");
        b.getPhotos().add(p);
        assertTrue("PhotoLibrary createAlbum did not create the specified album.", b.createAlbum("Vacation"));
        assertTrue("PhotoLibrary addPhotoToAlbum did not add a photo to an empty album", b.addPhotoToAlbum(p, "Vacation"));
        assertTrue("PhotoLibrary.removePhoto() failed (removePhoto() returned wrong result) (also check getPhotos())", b.removePhoto(new Photograph("Caption","Filename")));
        assertEquals("PhotoLibrary.removePhoto() failed (photo wasn't removed) (also check getPhotos())", new ArrayList<Photograph>(), b.getPhotos());
        Album a = null;
        for (Album c: b.getAlbums()) {
            if (c.getName().equals("Vacation"))
                a = c;
        }
        assertEquals("PhotoLibrary.removePhoto() failed (photo wasn't removed from the albums) (also check getAlbums())", new ArrayList<Photograph>(), a.getPhotos());

    }

    @Test(timeout=100)
    public void eraseThere() {
        PhotoLibrary b = new PhotoLibrary("Peleg", 17);
        b.getPhotos().add(new Photograph("C1", "F1"));
        b.getPhotos().add(new Photograph("C2", "F2"));
        b.getPhotos().add(new Photograph("C3", "F3"));
        assertTrue("PhotoLibrary createAlbum did not create the specified album.", b.createAlbum("Vacation"));
        assertTrue("PhotoLibrary addPhotoToAlbum did not add a photo to an empty album", b.addPhotoToAlbum(new Photograph("C2", "F2"), "Vacation"));
        assertTrue("PhotoLibrary createAlbum did not create the specified album.", b.createAlbum("Vacation2"));
        assertTrue("PhotoLibrary addPhotoToAlbum did not add a photo to an empty album", b.addPhotoToAlbum(new Photograph("C1", "F1"), "Vacation2"));
        assertTrue("PhotoLibrary addPhotoToAlbum did not add a photo to an empty album", b.addPhotoToAlbum(new Photograph("C2", "F2"), "Vacation2"));
        assertTrue("PhotoLibrary addPhotoToAlbum did not add a photo to an empty album", b.addPhotoToAlbum(new Photograph("C3", "F3"), "Vacation2"));

        assertTrue(b.removePhoto(new Photograph("C2","F2")));
        assertFalse("PhotoLibrary.removePhoto() failed (something wasn't removed) (also check getPhotos())", b.getPhotos().contains(new Photograph("C2","F2")));
        Album a = null;
        for (Album c: b.getAlbums()) {
            if (c.getName().equals("Vacation"))
                a = c;
        }
        assertEquals("PhotoLibrary.removePhoto() failed (photo wasn't removed from the albums) (also check getAlbums())", new ArrayList<Photograph>(), a.getPhotos());
        a = null;
        for (Album c: b.getAlbums()) {
            if (c.getName().equals("Vacation2"))
                a = c;
        }
        assertFalse("PhotoLibrary.removePhoto() failed (something wasn't removed from an album) (also check getPhotos())", a.getPhotos().contains(new Photograph("C2","F2")));
        assertTrue("PhotoLibrary.removePhoto() failed (removed too much from albums) (also check getPhotos())", a.getPhotos().contains(new Photograph("C1","F1")));
        assertTrue("PhotoLibrary.removePhoto() failed (removed too much from albums) (also check getPhotos())", a.getPhotos().contains(new Photograph("C3","F3")));
        assertEquals("PhotoLibrary removePhoto removed too much from albums", 2, a.getPhotos().size());
        assertTrue("PhotoLibrary.removePhoto() failed (removed too much) (also check getPhotos())", b.getPhotos().contains(new Photograph("C1","F1")));
        assertTrue("PhotoLibrary.removePhoto() failed (removed too much) (also check getPhotos())", b.getPhotos().contains(new Photograph("C3","F3")));
        assertEquals("PhotoLibrary removePhoto removed too much", 2, b.getPhotos().size());
    }

    @Test(timeout=100)
    public void testGetPhotosEdge() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        assertEquals("PhotoLibrary's inherited getPhotos(rating) did not return an empty list if no photos", new ArrayList<Photograph>(), a.getPhotos(2));

    }

    @Test(timeout=100)
    public void testGetPhotos() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("PhotoLibrary's inherited getPhotos(rating) did not return photos of rating 0 or higher", 3, a.getPhotos(0).size());
        assertTrue("PhotoLibrary's inherited getPhotos(rating) did not return correct photos of rating 0 or higher", a.getPhotos(0).contains(b));
        assertTrue("PhotoLibrary's inherited getPhotos(rating) did not return correct photos of rating 0 or higher", a.getPhotos(0).contains(c));
        assertTrue("PhotoLibrary's inherited getPhotos(rating) did not return correct photos of rating 0 or higher", a.getPhotos(0).contains(d));
    }


    @Test(timeout=100)
    public void testGetPhotos2() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("PhotoLibrary's inherited getPhotos(rating) did not return photos of rating 2 or higher", 2, a.getPhotos(2).size());
        assertTrue("PhotoLibrary's inherited getPhotos(rating) did not return correct photos of rating 2 or higher", a.getPhotos(2).contains(b));
        assertTrue("PhotoLibrary's inherited getPhotos(rating) did not return correct photos of rating 2 or higher", a.getPhotos(2).contains(c));
        assertEquals("PhotoLibrary's inherited getPhotos(rating) did not return photos of rating 5 or higher", 1, a.getPhotos(5).size());
        assertTrue("PhotoLibrary's inherited getPhotos(rating) did not return correct photos of rating 5 or higher", a.getPhotos(5).contains(c));
    }


    @Test(timeout=100)
    public void testGetPhotosYear() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("PhotoLibrary's inherited getPhotosInYear(year) did not return photos in year", 2, a.getPhotosInYear(2016).size());
        assertTrue("PhotoLibrary's inherited getPhotosInYear(year) did not return correct photos in year", a.getPhotosInYear(2016).contains(d));
        assertTrue("PhotoLibrary's inherited getPhotosInYear(year) did not return correct photos in year", a.getPhotosInYear(2016).contains(c));
    }

    @Test(timeout=100)
    public void testGetPhotosYear3() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 3);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("PhotoLibrary's inherited getPhotosInYear() returned photos when it shouldn't have", 0, a.getPhotosInYear(2019).size());
    }


    @Test(timeout=100)
    public void testGetPhotosMonth() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-09-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("PhotoLibrary's inherited getPhotosInMonth(Month) did not return photos in Month", 1, a.getPhotosInMonth(9, 2014).size());
        assertTrue("PhotoLibrary's inherited getPhotosInMonth(Month) did not return correct photos in Month", a.getPhotosInMonth(9, 2016).contains(d));
        assertFalse("PhotoLibrary's inherited getPhotosInMonth(Month) did not return correct photos in Month", a.getPhotosInMonth(9, 2016).contains(c));
        assertFalse("PhotoLibrary's inherited getPhotosInMonth(Month) did not return correct photos in Month", a.getPhotosInMonth(9, 2016).contains(b));
    }

    @Test(timeout=100)
    public void testGetPhotosMonth3() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 3);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("PhotoLibrary's inherited getPhotosInMonth() returned photos when it shouldn't have", 0, a.getPhotosInMonth(4, 2016).size());
        assertEquals("PhotoLibrary's inherited getPhotosInMonth() returned photos when it shouldn't have", 0, a.getPhotosInMonth(5, 2019).size());
    }

    @Test(timeout=100)
    public void testaddPhoto() {
        PhotoLibrary b = new PhotoLibrary("Arphaxad", 14);
        Photograph k = new Photograph("some title", "some author");
        assertTrue("PhotoLibrary's inherited addPhoto() failed", b.addPhoto(k));
        assertTrue("PhotoLibrary's inherited addPhoto() failed (photos list doesn't include added photo)", b.getPhotos().contains(k));
    }

    @Test(timeout=100)
    public void testGetPhotosBetween() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2016-09-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("PhotoLibrary's inherited getPhotosBetween() did not return photos in Between", 2, a.getPhotosBetween("2016-05-12", "2017-01-01").size());
        assertTrue("PhotoLibrary's inherited getPhotosBetween() did not return correct photos between good dates", a.getPhotosBetween("2016-05-12", "2017-01-01").contains(d));
        assertTrue("PhotoLibrary's inherited getPhotosBetween() did not return correct photos between good dates", a.getPhotosBetween("2016-05-12", "2017-01-01").contains(b));
        assertFalse("PhotoLibrary's inherited getPhotosBetween() did not return correct photos between good dates", a.getPhotosBetween("2016-05-12", "2017-01-01").contains(c));
    }


    @Test(timeout=100)
    public void testGetPhotosBetween2() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph b = new Photograph("mypic45.jpg", "Grand Canyon", "2014-01-11", 2);
        Photograph c = new Photograph("water.jpg", "Rafting", "2016-05-11", 5);
        Photograph d = new Photograph("water2.jpg", "Rafting 2", "2016-09-30", 1);

        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(b));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(c));
        assertTrue("PhotoLibrary's inherited addPhoto did not add a photo", a.addPhoto(d));
        assertEquals("PhotoLibrary's inherited getPhotosInBetween() returned photos when dates not formatted correctly", null, a.getPhotosBetween("2016-15-11", "2017-01-02"));
        assertEquals("PhotoLibrary's inherited getPhotosInBetween() returned photos when dates not formatted correctly", null, a.getPhotosBetween("2016-10-11", "2017-02-34"));
    }

    @Test(timeout=100)
    public void testCreateAlbum() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);

        assertTrue("PhotoLibrary createAlbum() did not create an album", a.createAlbum("My Album"));
        boolean contains = false;
        Album c = new Album("My Album");
        for (Album b : a.getAlbums()) {
            if (c.equals(b))
                contains = true;
        }
        assertTrue("PhotoLibrary createAlbum() did not create an album appropriately", contains);
        assertEquals("PhotoLibrary createAlbum() created too many albums", 1, a.getAlbums().size());
    }

    @Test(timeout=100)
    public void testRemoveAlbumEdge() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);

        assertFalse("PhotoLibrary removeAlbum() failed on an edge case", a.removeAlbum("My Album"));
    }

    @Test(timeout=100)
    public void testRemoveAlbum() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);

        assertTrue("PhotoLibrary createAlbum() did not create an album", a.createAlbum("My Album"));
        assertTrue("PhotoLibrary createAlbum() did not create an album", a.createAlbum("Vacation Photos"));
        assertTrue("PhotoLibrary createAlbum() did not create an album", a.createAlbum("Vacation Photos 2"));

        assertTrue("PhotoLibrary removeAlbum() failed to remove an album", a.removeAlbum("My Album"));
        boolean contains = false;
        Album c = new Album("My Album");
        for (Album b : a.getAlbums()) {
            if (c.equals(b))
                contains = true;
        }
        assertFalse("PhotoLibrary removeAlbum() failed to remove an album", contains);

        assertTrue("PhotoLibrary createAlbum() did not create an album", a.createAlbum("My Album"));
        contains = false;
        c = new Album("My Album");
        for (Album b : a.getAlbums()) {
            if (c.equals(b))
                contains = true;
        }
        assertTrue("PhotoLibrary createAlbum() did not allow creation of an album (that was previously removed)", contains);
        assertTrue("PhotoLibrary removeAlbum() failed to remove an album", a.removeAlbum("My Album"));
        contains = false;
        c = new Album("My Album");
        for (Album b : a.getAlbums()) {
            if (c.equals(b))
                contains = true;
        }
        assertFalse("PhotoLibrary removeAlbum() failed to remove an album", contains);
    }

    @Test(timeout=100)
    public void testAddPhotoToAlbum() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph p = new Photograph("Caption", "My Filename");

        assertTrue("PhotoLibrary createAlbum() did not create an album", a.createAlbum("My Album"));
        
        assertFalse("PhotoLibrary addPhotoToAlbum() allowed adding a photo that is not in the photo stream", a.addPhotoToAlbum(p, "My Album"));
        boolean contains = false;
        Album c = new Album("My Album");
        Album real = null;
        for (Album b : a.getAlbums()) {
            if (c.equals(b))
                real = b;
        }
        assertTrue("PhotoLibrary createAlbum() did not create an album", real != null);
        assertFalse("PhotoLibrary addPhotoToAlbum() allowed adding a photo that is not in the photo stream", real.hasPhoto(p));        
    }


    @Test(timeout=100)
    public void testRemovePhotoFromAlbum2() {
        PhotoLibrary a = new PhotoLibrary("MyLibrary", 1);
        Photograph p = new Photograph("Caption", "My Filename");
        a.addPhoto(p);
        assertTrue("PhotoLibrary createAlbum() did not create an album", a.createAlbum("My Album"));
        a.addPhotoToAlbum(p, "My Album");

        assertTrue("PhotoLibrary removePhotoFromAlbum() did not remove a photo from an album correctly", a.removePhotoFromAlbum(p, "My Album"));  boolean contains = false;
        Album c = new Album("My Album");
        Album real = null;
        for (Album b : a.getAlbums()) {
            if (c.equals(b))
                real = b;
        }
        assertTrue("PhotoLibrary createAlbum() did not create an album", real != null);
        assertFalse("PhotoLibrary removePhotoFromAlbum() did not correctly remove the photo from an album", real.hasPhoto(p));      
    }

    @Test(timeout=100)
    public void equalsSimilar() {
        PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
        PhotoLibrary c = new PhotoLibrary("Le Petit Prince", 42);
        assertTrue("PhotoLibrary.equals() failed: Same id and name", b.equals(c));
    }

    @Test(timeout=100)
    public void equalsDifferentNames() {
        PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
        PhotoLibrary c = new PhotoLibrary("The Little Prince", 42);
        assertTrue("PhotoLibrary.equals() failed: Same id, different names", b.equals(c));
    }
 
    
    @Test(timeout=100)
    public void equalsDifferentTypes() throws Exception {
        try {
            PhotoLibrary b = new PhotoLibrary("Le Petit Prince", 42);
            assertFalse("PhotoLibrary.equals() failed: Compare PhotoLibrary to Integer", b.equals(42));
        } catch (ClassCastException e) {
            fail("PhotoLibrary.equals() failed: Casting a non-PhotoLibrary to a PhotoLibrary");
        } catch (Exception e) {
            throw e;
        }
    }

    @Test(timeout=100)
    public void testToString() {
        PhotoLibrary b = new PhotoLibrary("Grand Canyon Photos", 402);
        b.getPhotos().add(new Photograph("Photo1", "File1.jpg"));
        b.createAlbum("My New Album");
        b.createAlbum("Water Pics");
        String s = b.toString();
        assertTrue("PhotoLibrary.toString must have name", s.contains("Grand Canyon Photos"));
        assertTrue("PhotoLibrary.toString must have id", s.contains("402"));
        assertTrue("PhotoLibrary.toString must have list of Photographs (also check getPhotos())", s.contains("Photo1"));
        assertTrue("PhotoLibrary.toString must have list of Photographs (also check getPhotos())", s.contains("File1.jpg"));
        assertTrue("PhotoLibrary.toString must have album names", s.contains("My New Album"));
        assertTrue("PhotoLibrary.toString must have album names", s.contains("Water Pics"));
    }

}

